// Copyright (C) 2013 - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 2008-2009 - INRIA - Michael Baudin
// Copyright (C) 1986 - Bennett Fox
//
// This file must be used under the terms of the GNU LGPL license.

function [nsim,skip,leap] = lowdisc_fauresuggest ( varargin )
  // Returns favorable parameters for Faure sequences.
  //
  // Calling Sequence
  //   [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base )
  //   [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base , nsimmin )
  //
  // Parameters
  //    dim : a floating point integer, the spatial dimension.
  //    base : a floating point integer, the base used in Faure's sequence.
  //    nsimmin : a floating point integer, the minimum required number of simulations. Default nsimmin = 1.
  //    nsim : a floating point integer, the number of simulations to perform, with nsim >= nsimmin.
  //    skip : a floating point integer, the number of initial elements to skip in the sequence.
  //    leap : a floating point integer, the number of elements to ignore each time an element is generated.
  //
  // Description
  //    This routine provides favorable parameters to be used with a Faure sequence.
  //    
  //    We use suggestions from :
  //    "Algorithm 647: Implementation and Relative Efficiency of Quasirandom
  //    Sequence Generators", B. L. Fox, ACM Transactions on Mathematical Software, 
  //    Volume 12, Number 4, pages 362-376, 1986.
  //    
  //    The citation which governs the choice for the <literal>skip</literal> parameter 
  //    is the following.
  //    "For QS greater than a half-dozen or so, one sees that the Faure 
  //    sequence starts out as a series of points lying on slightly warped,
  //    nearly parallel hyperplanes with several points unduly close to (0,0,...,0)
  //    or (1,1,...,1). Our first two preliminary versions of the code initialized 
  //    NEXTN to 0 (following [5]) and to 1, both leading to terrible results
  //    at s=20 and s=25. Deleting the first QS^4-2 terms, for example, eliminates these 
  //    effects because of the scrambling then generated by the weigted sums in Faure's 
  //    scheme."
  //
  //    With respect to the number of simulations <literal>nsim</literal>, 
  //    the citation p. 647 is the following.
  //    "Sobol' says (personal communication, 1985) that taking N equal to an integral
  //    power of qs favors the generator Faure, apparently because then again D*(N)=O((log(N)^(s-1)) [...]."
  //    And p. 374 :
  //    "[...] otherwise, use Faure and check for stability at N equal to successive 
  //    powers of qs, starting at N = qs^3."
  //
  //    We return skip = b^4 - 2.
  //    We compute k = max( 3 , ceil(log(nsimmin)/log(b)) ) so that the 
  //    number of simulations is greater than nsimmin. 
  //    This ensures good equidistribution of at least the three dimensionnal projections 
  //    of the sequence. 
  //    We compute nsim = b^k.
  //    We return leap = 0.
  //
  //    We might also be interested by the experiments in :
  //    "Computational investigations of low-discrepancy sequences", Kocis,
  //    L. and Whiten, W. J. 1997. ACM Trans. Math. Softw. 23, 2 (Jun. 1997),
  //    266-294. Especially p. 277.
  //    The citation p. 277 is the following.
  //    "Properties of the Faure sequence leaped were studied experimentally
  //    using the Fox implementation for s = 40 with base b = 41 and 
  //    leaps 2 to 31. Observation of the projections of the points of the Faure
  //    sequence leaped in two dimensional planes selected at random demonstrated
  //    a considerable improvement on the original Faure sequence. Any value of the leap
  //    (except L = b = 41) improved the Faure sequence. The best results
  //    were achieved for leaps L = 5, 6, 12 to 17 and then around 30."
  //    This suggestions are not implemented in <literal>lowdisc_fauresuggest</literal>.
  //
  // Examples
  //
  // // See the minimum number of simulations for integration in dimension 4.
  // dim = 4;
  // base = 5;
  // // See "Algorithm 659", p97, Table II
  // [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base );
  // [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base , 10000 );
  // [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base , 50000 );
  // [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base , 100000 );
  //
  // // Use the minimum number of simulations for integration in dimension 4.
  // // Use Faure
  // dim = 4;
  // nsimmin = 1000;
  // lds = lowdisc_new("faure");
  // lds = lowdisc_configure(lds,"-dimension",dim);
  // base = lowdisc_get(lds,"-faureprime");
  // [nsim,skip,leap] = lowdisc_fauresuggest ( dim , base , nsimmin );
  // lds = lowdisc_configure(lds,"-skip",skip);
  // lds = lowdisc_configure(lds,"-leap",leap);
  // lds = lowdisc_startup (lds);
  // [lds,experiments]=lowdisc_next(lds,nsim);
  // lds = lowdisc_destroy(lds);
  // 
  //  Authors
  //   Michael Baudin - 2010 - DIGITEO
  //   Bennett Fox - 1986
  //

  // check the number of input arguments
  [lhs,rhs]=argn();
  if ( rhs<2 | rhs>3 ) then
    error(msprintf(gettext("%s: Wrong number of input argument(s): %d or %d expected.\n"),"lowdisc_fauresuggest",2,3))
  end
  // Get arguments
  dim = varargin(1)
  base = varargin(2)
  nsimmin = 1
  if ( rhs >= 3 ) then
    if ( varargin(3) <> [] ) then
      nsimmin = varargin(3)
    end
  end
  // Check dim
  if ( dim < 1 ) then
    errmsg = msprintf(gettext("%s: The dim argument should be greater than 1, but is equal to %d."), "lowdisc_fauresuggest", dim);
    error(errmsg)
  end
  // Check base
  if ( base < 1 ) then
    errmsg = msprintf(gettext("%s: The base argument should be greater than 1, but is equal to %d."), "lowdisc_fauresuggest", base);
    error(errmsg)
  end
  // Check nsimmin
  if ( nsimmin < 1 ) then
    errmsg = msprintf(gettext("%s: The nsimmin argument should be greater than 1, but is equal to %d."), "lowdisc_fauresuggest", nsimmin);
    error(errmsg)
  end
  // Compute nsim
  k = max ( [ 3 , ceil(log(nsimmin)/log(base)) ] )
  nsim = base^k
  // Compute skip and leap
  skip = base^4 - 2
  leap = 0
endfunction

