﻿// Copyright (C) 2013 - Michael Baudin
// Copyright (C) 2005-2007 - John Burkardt
// Copyright (C) 2009-2010 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license
// http://www.gnu.org/copyleft/lesser.html

#ifndef _LOWDISC_HALTON_D_H_
#define _LOWDISC_HALTON_D_H_

#ifdef _MSC_VER
#if LIBLOWDISC_EXPORTS 
#define LOWDISC_IMPORTEXPORT __declspec (dllexport)
#else
#define LOWDISC_IMPORTEXPORT __declspec (dllimport)
#endif
#else
#define LOWDISC_IMPORTEXPORT
#endif

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS


	// The scrambling methods
	// No scrambling
#define HALTON_SCRAMBLINGZERO 1
	// Kocis - Whiten scrambling
#define HALTON_SCRAMBLINGRR2 2
	// Vandewoestyne and Cools "Reverse Halton" scrambling
#define HALTON_SCRAMBLINGREVERSE 3


	//! Class of Scrambled Halton Sequence
class LOWDISC_IMPORTEXPORT Halton {
public:


	// Halton --
	// startup the sequence.
	//	
	//  Parameters:
	//
	//    * dim_num, the spatial dimension, 
	//      Default: dim_num = 1;
	//      Required: 1 <= dim_num is required.
	//
	//    * seed[0,1,...,dim_num-1], the Halton sequence element corresponding to index = 0.
	//      Default seed = (0, 0, ... 0).  
	//      Required: 0 <= seed[0,1,...,dim_num-1].
	//
	//    * leap[0,1,...,dim_num-1], the succesive jumps in the Halton sequence.
	//      Default: leap = (1, 1, ..., 1). 
	//      Required: 1 <= leap[0,1,...,dim_num-1].
	//
	//    * base[0,1,...,dim_num-1], the Halton bases.
	//      Default: base = (2, 3, 5, 7, 11, ... ). 
	//      If base[i] = 0 then the prime number #i is automatically set.
	//	    If base[i] > 1 then the base is directly used.
	//	    If base[i] <0 or base[i] = 1, this is an error.
	//
	//    * scrambling, the scrambling method
	//      halton_scramblingZero (i.e. no scrambling) or halton_scramblingRR2
	//
	Halton ( int dim_num , int base[] , int seed[] , int leap[] , int scrambling);

	// halton_stop --
	// shutdown the sequence.
	//
	// Description
	//  Deletes the unnecessary memory.
	~Halton ( );

	// next --
	//   Computes the next element in a leaped Halton subsequence.
	//
	// Parameters
	//
	//    * index, the subsequence index.
	//      Default: index = 0.
	//      Required: 0 <= index.
	//
	//    * Output, double r[0,1,...,dim_num-1], the next element of the leaped Halton
	//    subsequence.
	//
	//  Discussion:
	//
	//    The dim_num-dimensional Halton sequence is really dim_num separate
	//    sequences, each generated by a particular base.
	//
	//    This routine selects elements of a "leaped" subsequence of the 
	//    Halton sequence.  The subsequence elements are indexed by a
	//    quantity called index, which starts at 0.  The index-th subsequence 
	//    element is simply element 
	//
	//      seed[0,1,...,dim_num-1] + index * leap[0,1,...,dim_num-1]
	//
	//    of the original Halton sequence.
	//
	//    Call halton_start() before calling halton_next.
	// Example
	//  With dim_num=3, leap[i]=1 (for i=1,2,3), base=[2,3,5].
	//  No scrambling.
	//  0:   0.        0.           0.   
	//  1:   0.5       0.3333333    0.2   
	//  2:   0.25      0.6666667    0.4   
	//  3:   0.75      0.1111111    0.6   
	//  4:   0.125     0.4444444    0.8   
	//  5:   0.625     0.7777778    0.04  
	//  6:   0.375     0.2222222    0.24  
	//  7:   0.875     0.5555556    0.44  
	//  8:   0.0625    0.8888889    0.64  
	//  9:   0.5625    0.0370370    0.84  
	// 10:   0.3125    0.3703704    0.08  
	//
	// Example:
	//  With dim_num=3, leap[i]=1 (for i=1,2,3), base=[2,3,5].
	//  RR2 scrambling.
	//  0:   0.        0.           0.   
	//  1:   0.5       0.6666667    0.8   
	//  2:   0.25      0.3333333    0.4   
	//  3:   0.75      0.2222222    0.2   
	//  4:   0.125     0.8888889    0.6   
	//  5:   0.625     0.5555556    0.16  
	//  6:   0.375     0.1111111    0.96  
	//  7:   0.875     0.7777778    0.56  
	//  8:   0.0625    0.4444444    0.36  
	//  9:   0.5625    0.0740741    0.76  
	// 10:   0.3125    0.7407407    0.08
	void next ( int index , double r[] );

	// dim_num_get --
	// gets the spatial dimension for a leaped Halton subsequence.
	//
	//  Parameters:
	//    Output, int HALTON_DIM_NUM_GET, the spatial dimension.
	int dim_num_get ( void );

	// base_get --
	// gets the base vector for a leaped Halton subsequence.
	//
	//  Parameters:
	//    Output, int base[], the bases
	void base_get ( int base[] );

	// leap_get --
	// gets the leap vector for a leaped Halton subsequence.
	//
	//  Parameters:
	//    Output, int leap[], the successive jumps in the Halton sequence.
	void leap_get ( int leap[] );

	// seed_get --
	// gets the seed vector for a leaped Halton subsequence.
	//
	//  Parameters:
	//    Output, int seed[], the sequence index corresponding to STEP = 0.
	void seed_get ( int seed[] );

	// step_get 
	// gets the step for the leaped Halton subsequence.
	int step_get ( void );

	// scrambling_get --
	// get the scrambling option.
	//
	//  Parameters:
	//    Output, int scrambling: halton_scramblingZero or halton_scramblingRR2
	int scrambling_get (  );
private:
	//
	//  These variables are accessible to the user via calls to set/get routines.
	//

	// The number of dimensions
	int  halton_dim_num;
	// The bases, an array[0,1,...,dim_num-1]
	// halton_base[i] is a prime number, for i=0,1,...,dim_num
	int *halton_base;
	// The leap, an array[0,1,...,dim_num-1]
	// halton_leap[i]>=1, for i=0,1,...,dim_num
	int *halton_leap;
	// The seed, an array[0,1,...,dim_num-1]
	// halton_seed[i]>=0, for i=0,1,...,dim_num
	int *halton_seed;

	// The scrambling method
	int halton_scrambling;

	// The permutation used in the scrambling
	int ** halton_sigma;
private:
	//
	// Private methods
	//

	// vandercorput --
	// Returns the index-th element of the Van Der Corput 
	// sequence in base b
	//
	// Parameters
	//   index, input : the index in the sequence, index>=0
	//   base, input : the base of the sequence, base>=2 (base must be a prime number)
	//   result, output : the index-th element in the sequence, in [0,1].
	//
	// Example:
	// base=2;
	// index=0, r=0.  
	// index=1, r=0.5  
	// index=2, r=0.25  
	// index=3, r=0.75  
	// index=4, r=0.125  
	// index=5, r=0.625  
	// index=6, r=0.375  
	// index=7, r=0.875  
	// index=8, r=0.0625  
	// index=9, r=0.5625  
	// 
	double vandercorput(int index, int base);

	// halton_RR2Scrambling --
	// Compute RR2 scrambling.
	//
	// Parameters
	// dim_num : the number of dimensions, dim_num>=1
	// i : the dimension to scramble, 1<=i<=dim_num
	// base : an array[0,1,...,dim_num-1], the base for direction i
	// sigma : an array, the permutations for all directions
	// sigma[0] : an array0[0,1,...,base[0]], the permutation for direction 1
	// sigma[1] : an array0[0,1,...,base[1]], the permutation for direction 1
	// ...
	// sigma[dim_num-1] : an array0[0,1,...,base[dim_num-1]], the permutation for direction dim_num
	//
	// Reference
	// L. Kocis and W. Whiten. Computational investigations of 
	// low discrepancy sequences.
	// ACM Trans. Mathematical Software, 23:266–294, 1997.
	//
	// Example
	// // Reference
	// Generalized Halton Sequences in 2008: 
	// A Comparative Study
	// HENRI FAURE
	// CHRISTIANE LEMIEUX
	// Section 3. OVERVIEW OF PROPOSED GENERALIZED HALTON SEQUENCES
	// Item (1) KW [Kocis and Whiten 1997]:
	// For instance, take s = 3. 
	// Then n3 = ceil(log 5/log 2) = 3
	// and so σ corresponds to the permutation [0,4,2,6,1,5,3,7]. 
	// Hence σ1 corresponds to [0,1], σ2 to [0,2,1] 
	// and σ3 to [0,4,2,1,3].
	// [Note From MB : fixed error in the permutation - switched 
	// 5 and 3].
	void RR2Scrambling();

	// ReverseScrambling --
	// Compute the Reverse Scrambling of Vandewoestyne and Cools
	// If digit==0, then sigma[digit]=0, 
	// else sigma[digit]=base-digit.
	//
	// This is :
	// 0 1
	// 0 2 1
	// 0 5 4 3 2 1
	// 0 7 6 5 4 3 2 1
	void ReverseScrambling();

	// scrambledVDC --
	// Returns the index-th element of a scrambled Van Der Corput 
	// sequence in base b
	//
	// Parameters
	//   index, input : the index in the sequence, index>=0
	//   base, input : the base of the sequence, base>=2 (base must be a prime number)
	//   sigma, input : an array[0,1,...,base-1], the permutation of the digits, 0<=sigma[i]<=base-1.
	//   result, output : the index-th element in the sequence, in [0,1].
	//
	// Example:
	// sigma=[0,2,1]
	// base=3
	// index=1, r=0.6666667
	// index=2, r=0.3333333
	// index=3, r=0.2222222
	// index=4, r=0.8888889
	// index=5, r=0.5555556
	// index=6, r=0.1111111
	// index=7, r=0.7777778
	// index=8, r=0.4444444
	// index=9, r=0.0740741
	// 
	double scrambledVDC(int index, int base, int * sigma);

};

__END_DECLS


#endif /* _LOWDISC_HALTON_D_H_ */

